# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-08 05:07
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_courier', '0008_notification_required'),
    ]

    operations = [
        migrations.AddField(
            model_name='template',
            name='subject',
            field=models.CharField(
                blank=True, max_length=500, verbose_name='subject'),
        ),
        migrations.AlterField(
            model_name='template',
            name='backend',
            field=models.CharField(max_length=100, verbose_name='backend'),
        ),
        migrations.AlterField(
            model_name='template',
            name='content',
            field=models.TextField(verbose_name='content'),
        ),
        migrations.AlterField(
            model_name='template',
            name='is_active',
            field=models.BooleanField(
                default=True,
                help_text='When not active, the template will be ignored',
                verbose_name='is active'),
        ),
        migrations.AlterField(
            model_name='template',
            name='target',
            field=models.CharField(
                default='re',
                help_text='Who this message actually gets sent to.',
                max_length=103, verbose_name='target'),
        ),
    ]
