{% extends "admin/change_form.html" %}
{% load i18n admin_urls %}
{% block object-tools-items %}
{% if original.can_issue_custom %}
<li>
    <a href="../issue/">{% trans "Send Notification" %}</a>
</li>
{% endif %}
<li>
    {% url opts|admin_urlname:'history' original.pk|admin_urlquote as history_url %}
    <a href="{% add_preserved_filters history_url %}" class="historylink">
        {% trans "History" %}</a>
</li>
{% endblock %}
