/* -------------------------------------------------------------------
// markItUp!
// By Jay Salvat - http://markitup.jaysalvat.com/
// ------------------------------------------------------------------*/
.markItUp .button-b a {
	background-image:url(images/bold.png);
}
.markItUp .button-i a {
	background-image:url(images/italic.png);
}
.markItUp .button-s a {
	background-image:url(images/stroke.png);
}
.markItUp .button-ul a {
	background-image:url(images/list-bullet.png); 
}
.markItUp .button-ol a {
	background-image:url(images/list-numeric.png);
}

.markItUp .button-img a {
	background-image:url(images/picture.png); 
}
.markItUp .button-a a {
	background-image:url(images/link.png);
}
.markItUp .button-q a {
	background-image:url(images/quotes.png);
}
.markItUp .button-code a {
	background-image:url(images/code.png);
}
.markItUp .preview a {
	background-image:url(images/preview.png);
}
